/*
 * #%L
 * EUGene :: EUGene
 * 
 * $Id: EugeneTagValues.java 1031 2011-01-21 17:22:03Z tchemit $
 * $HeadURL: http://svn.nuiton.org/svn/eugene/tags/eugene-2.3.5/eugene/src/main/java/org/nuiton/eugene/EugeneTagValues.java $
 * %%
 * Copyright (C) 2004 - 2010 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */
package org.nuiton.eugene;

import org.nuiton.eugene.java.JavaBeanTransformer;
import org.nuiton.eugene.java.JavaGeneratorUtil;
import org.nuiton.eugene.models.object.ObjectModel;
import org.nuiton.eugene.models.object.ObjectModelClassifier;
import org.nuiton.eugene.models.object.ObjectModelElement;
import org.nuiton.eugene.models.object.ObjectModelReader;

import static org.nuiton.eugene.ModelPropertiesUtil.TagValueDefinition;

/**
 * Defines all tag values managed by Eugene.
 * <p/>
 * In another library using eugene, please extends this contract to put your
 * own tag values, to get a unique place where to find tag values.
 *
 * @author tchemit <chemit@codelutin.com>
 * @since 2.3
 */
public interface EugeneTagValues {

    /**
     * Tag value to add the version of the model from outside (says in the
     * properties file associated to the model)..
     * <p/>
     * Actually, the eugene api does not use to modify the model. It is only
     * used while reading the properties associated with a model and if found is
     * directly set to the {@code version} field of the model.
     *
     * @see ObjectModelReader#loadModelTagValue(ObjectModel, String, String)
     * @since 2.3
     */
    @TagValueDefinition(target = {ObjectModel.class},
                        documentation = "To set the version of the model.")
    String TAG_VERSION = "version";

    /**
     * Tag value to add on constants enumeration (or other incoming dev)
     * a prefix to constant to generate.
     * <p/>
     * You can globaly use it on the complete model or to a specific classifier.
     *
     * @see GeneratorUtil#getDocumentationTagValue(ObjectModel)
     * @see GeneratorUtil#getDocumentationTagValue(ObjectModelElement)
     * @since 2.5
     */
    @TagValueDefinition(target = {ObjectModel.class, ObjectModelElement.class},
                        documentation = "Sets the documentation of a model or any of his elements")
    String TAG_DOCUMENTATION = "documentation";

    /**
     * Tag value to add on constants enumeration (or other incoming dev)
     * a prefix to constant to generate.
     * <p/>
     * You can globaly use it on the complete model or to a specific classifier.
     *
     * @see JavaGeneratorUtil#getConstantPrefixTagValue(ObjectModel, ObjectModelClassifier)
     * @since 2.3
     */
    @TagValueDefinition(target = {ObjectModel.class, ObjectModelClassifier.class},
                        documentation = "Sets the prefix of any constant to be generated for the given class")
    String TAG_CONSTANT_PREFIX = "constantPrefix";

    /**
     * Tag value to set if do not want any pcs (says PropertyChangeSupport in a generated bean).
     * <p/>
     * You can globaly use it on the complete model or to a specific classifier.
     *
     * @see JavaBeanTransformer
     * @see JavaGeneratorUtil#getNoPCSTagValue(ObjectModel, ObjectModelClassifier)
     * @since 2.3
     */
    @TagValueDefinition(target = {ObjectModel.class, ObjectModelClassifier.class},
                        documentation = "To specify to not generate any propertyChange code for a class or any class of a model")
    String TAG_NO_PCS = "noPCS";

    /**
     * Tag value to specify the i18n prefix to use whene generating i18n keys.
     * <p/>
     * You can globaly use it on the complete model or to a specific classifier.
     *
     * @see JavaGeneratorUtil#getI18nPrefixTagValue(ObjectModelElement, ObjectModel)
     * @since 2.3
     */
    @TagValueDefinition(target = {ObjectModel.class, ObjectModelClassifier.class},
                        documentation = "Sets the i18n prefix to use on I18n keys generated")
    String TAG_I18N_PREFIX = "i18n";
}
